# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import List, Optional

try:
    from pydantic.v1 import (
        BaseModel,
        Field,
        StrictBool,
        StrictStr,
        conint,
        conlist,
        validator,
    )
except ImportError:
    from pydantic import (
        BaseModel,
        Field,
        StrictBool,
        StrictStr,
        conint,
        conlist,
        validator,
    )

from mozart_api.models.latency_profile import LatencyProfile


class RoomCompensationProperties(BaseModel):
    """
    RoomCompensationProperties
    """

    value: Optional[StrictStr] = Field(
        default=None,
        description="The type of roomcompensation used / to use. simple:   For speakers without external speakers. advanced:   For products with external speakers or other advanced multichannel capabilities.   This requires that at least action in the other properties of RoomCompensationProperties is set. ",
    )
    action: Optional[StrictStr] = Field(
        default=None,
        description="Must be set if room compensation type is advanced. runAll:   Do measurements on all connected speakers. continue:   Continue from and including the speaker where last interrupted (stopped or failed). useSpeakerList:   Do measurements on the speakers in the list property. ",
    )
    continue_on_error: Optional[StrictBool] = Field(
        default=None,
        alias="continueOnError",
        description="On failing measurement on a speaker, default behavior is to stop measurement and skip the remaining speakers. Setting continueOnError to true will make the measurement process continue and finish measurement on all speakers, even though an error ocurred on one of the speakers. ",
    )
    latency_profile: Optional[LatencyProfile] = Field(
        default=None, alias="latencyProfile"
    )
    skip_automatic_role_assignment: Optional[StrictBool] = Field(
        default=None,
        alias="skipAutomaticRoleAssignment",
        description="Skip calculation of automatic role assignment.",
    )
    speaker_list: Optional[conlist(StrictStr)] = Field(
        default=None,
        alias="speakerList",
        description="List of speaker IDs to include in room compensation / automatic role assignment measurement. Relevant e.g. if you want to create a speaker group without the external speakers included. This can not be used for doing measurements on a partial speaker group, only on all speakers in an existing group or for all speakers in a new group. ",
    )
    speaker_preset: Optional[conint(strict=True, le=255, ge=0)] = Field(
        default=None,
        alias="speakerPreset",
        description="The Powerlink preset to use for the external PL/WPL speakers.",
    )
    __properties = [
        "value",
        "action",
        "continueOnError",
        "latencyProfile",
        "skipAutomaticRoleAssignment",
        "speakerList",
        "speakerPreset",
    ]

    @validator("value")
    def value_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "simple",
            "advanced",
        ):
            raise ValueError("must be one of enum values ('simple', 'advanced')")
        return value

    @validator("action")
    def action_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "runAll",
            "continue",
            "useSpeakerList",
        ):
            raise ValueError(
                "must be one of enum values ('runAll', 'continue', 'useSpeakerList')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoomCompensationProperties:
        """Create an instance of RoomCompensationProperties from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of latency_profile
        if self.latency_profile:
            _dict["latencyProfile"] = self.latency_profile.to_dict()
        # set to None if action (nullable) is None
        # and __fields_set__ contains the field
        if self.action is None and "action" in self.__fields_set__:
            _dict["action"] = None

        # set to None if continue_on_error (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.continue_on_error is None
            and "continue_on_error" in self.__fields_set__
        ):
            _dict["continueOnError"] = None

        # set to None if skip_automatic_role_assignment (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.skip_automatic_role_assignment is None
            and "skip_automatic_role_assignment" in self.__fields_set__
        ):
            _dict["skipAutomaticRoleAssignment"] = None

        # set to None if speaker_list (nullable) is None
        # and __fields_set__ contains the field
        if self.speaker_list is None and "speaker_list" in self.__fields_set__:
            _dict["speakerList"] = None

        # set to None if speaker_preset (nullable) is None
        # and __fields_set__ contains the field
        if self.speaker_preset is None and "speaker_preset" in self.__fields_set__:
            _dict["speakerPreset"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoomCompensationProperties:
        """Create an instance of RoomCompensationProperties from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoomCompensationProperties.parse_obj(obj)

        _obj = RoomCompensationProperties.parse_obj(
            {
                "value": obj.get("value"),
                "action": obj.get("action"),
                "continue_on_error": obj.get("continueOnError"),
                "latency_profile": LatencyProfile.from_dict(obj.get("latencyProfile"))
                if obj.get("latencyProfile") is not None
                else None,
                "skip_automatic_role_assignment": obj.get(
                    "skipAutomaticRoleAssignment"
                ),
                "speaker_list": obj.get("speakerList"),
                "speaker_preset": obj.get("speakerPreset"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel
except ImportError:
    from pydantic import BaseModel

from mozart_api.models.playback_content_metadata import PlaybackContentMetadata
from mozart_api.models.playback_progress import PlaybackProgress
from mozart_api.models.rendering_state import RenderingState
from mozart_api.models.source import Source


class PlaybackState(BaseModel):
    """
    PlaybackState
    """

    metadata: Optional[PlaybackContentMetadata] = None
    progress: Optional[PlaybackProgress] = None
    source: Optional[Source] = None
    state: Optional[RenderingState] = None
    __properties = ["metadata", "progress", "source", "state"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PlaybackState:
        """Create an instance of PlaybackState from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict["metadata"] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of progress
        if self.progress:
            _dict["progress"] = self.progress.to_dict()
        # override the default output from pydantic by calling `to_dict()` of source
        if self.source:
            _dict["source"] = self.source.to_dict()
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict["state"] = self.state.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PlaybackState:
        """Create an instance of PlaybackState from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PlaybackState.parse_obj(obj)

        _obj = PlaybackState.parse_obj(
            {
                "metadata": PlaybackContentMetadata.from_dict(obj.get("metadata"))
                if obj.get("metadata") is not None
                else None,
                "progress": PlaybackProgress.from_dict(obj.get("progress"))
                if obj.get("progress") is not None
                else None,
                "source": Source.from_dict(obj.get("source"))
                if obj.get("source") is not None
                else None,
                "state": RenderingState.from_dict(obj.get("state"))
                if obj.get("state") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import io
import re  # noqa: F401
import warnings

try:
    from pydantic.v1 import ValidationError, validate_arguments
except ImportError:
    from pydantic import ValidationError, validate_arguments

from typing import Awaitable, Optional, Union, overload

from typing_extensions import Annotated

try:
    from pydantic.v1 import Field, conint
except ImportError:
    from pydantic import Field, conint


from typing import Dict, List

from mozart_api.api_client import ApiClient
from mozart_api.api_response import ApiResponse
from mozart_api.exceptions import ApiTypeError, ApiValueError  # noqa: F401
from mozart_api.models.bass import Bass
from mozart_api.models.directivity import Directivity
from mozart_api.models.listening_mode import ListeningMode
from mozart_api.models.listening_mode_ref import ListeningModeRef
from mozart_api.models.loudness import Loudness
from mozart_api.models.sound_feature_set import SoundFeatureSet
from mozart_api.models.treble import Treble
from mozart_api.models.volume_level import VolumeLevel
from mozart_api.models.volume_mute import VolumeMute
from mozart_api.models.volume_settings import VolumeSettings
from mozart_api.models.volume_state import VolumeState


class SoundApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    async def activate_listening_mode(
        self, id: conint(strict=True, ge=0), **kwargs
    ) -> None:  # noqa: E501
        """Activate a listening mode  # noqa: E501


        :param id: (required)
        :type id: int
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the activate_listening_mode_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.activate_listening_mode_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    async def activate_listening_mode_with_http_info(
        self, id: conint(strict=True, ge=0), **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Activate a listening mode  # noqa: E501


        :param id: (required)
        :type id: int
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["id"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method activate_listening_mode" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params["id"] is not None:
            _path_params["id"] = _params["id"]

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/listening-modes/{id}/activate",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def get_active_listening_mode(self, **kwargs) -> ListeningModeRef:  # noqa: E501
        """Get the active listening mode  # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ListeningModeRef
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_active_listening_mode_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_active_listening_mode_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def get_active_listening_mode_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the active listening mode  # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ListeningModeRef, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_active_listening_mode" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "ListeningModeRef",
        }

        return await self.api_client.call_api(
            "/api/v1/sound/listening-modes/active",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def get_current_volume(self, **kwargs) -> VolumeState:  # noqa: E501
        """Get the current volume state  # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: VolumeState
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_current_volume_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_current_volume_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def get_current_volume_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the current volume state  # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(VolumeState, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_current_volume" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "VolumeState",
            "400": "ErrorModel",
        }

        return await self.api_client.call_api(
            "/api/v1/sound/volume",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def get_listening_mode_set(self, **kwargs) -> List[ListeningMode]:  # noqa: E501
        """Get available listening modes that match the current role (standalone, multichannel or both(all)).   # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: List[ListeningMode]
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_listening_mode_set_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_listening_mode_set_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def get_listening_mode_set_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Get available listening modes that match the current role (standalone, multichannel or both(all)).   # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(List[ListeningMode], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_listening_mode_set" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "List[ListeningMode]",
        }

        return await self.api_client.call_api(
            "/api/v1/sound/listening-modes",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def get_sound_features(self, **kwargs) -> Dict[str, SoundFeatureSet]:  # noqa: E501
        """Get available sound features that match the current role (standalone or multichannel).   # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Dict[str, SoundFeatureSet]
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_sound_features_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_sound_features_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def get_sound_features_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Get available sound features that match the current role (standalone or multichannel).   # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Dict[str, SoundFeatureSet], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_sound_features" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "Dict[str, SoundFeatureSet]",
        }

        return await self.api_client.call_api(
            "/api/v1/sound/features",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def get_volume_settings(self, **kwargs) -> VolumeSettings:  # noqa: E501
        """Get the current volume settings for default and maximum volume level  # noqa: E501


        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: VolumeSettings
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_volume_settings_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.get_volume_settings_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    async def get_volume_settings_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Get the current volume settings for default and maximum volume level  # noqa: E501


        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(VolumeSettings, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = []
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_volume_settings" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "VolumeSettings",
            "400": "ErrorModel",
        }

        return await self.api_client.call_api(
            "/api/v1/sound/volume/settings",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_current_volume_level(
        self,
        volume_level: Annotated[
            VolumeLevel, Field(..., description="The volume level to apply")
        ],
        **kwargs,
    ) -> None:  # noqa: E501
        """Set the current volume level  # noqa: E501


        :param volume_level: The volume level to apply (required)
        :type volume_level: VolumeLevel
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_current_volume_level_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_current_volume_level_with_http_info(
            volume_level, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_current_volume_level_with_http_info(
        self,
        volume_level: Annotated[
            VolumeLevel, Field(..., description="The volume level to apply")
        ],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Set the current volume level  # noqa: E501


        :param volume_level: The volume level to apply (required)
        :type volume_level: VolumeLevel
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["volume_level"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_current_volume_level" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["volume_level"] is not None:
            _body_params = _params["volume_level"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/volume/level",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_sound_settings_adjustments_bass(
        self, bass: Annotated[Bass, Field(..., description="the bass level")], **kwargs
    ) -> None:  # noqa: E501
        """Update the sound settings adjustments for bass  # noqa: E501


        :param bass: the bass level (required)
        :type bass: Bass
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_sound_settings_adjustments_bass_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_sound_settings_adjustments_bass_with_http_info(
            bass, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_sound_settings_adjustments_bass_with_http_info(
        self, bass: Annotated[Bass, Field(..., description="the bass level")], **kwargs
    ) -> ApiResponse:  # noqa: E501
        """Update the sound settings adjustments for bass  # noqa: E501


        :param bass: the bass level (required)
        :type bass: Bass
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["bass"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_sound_settings_adjustments_bass" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["bass"] is not None:
            _body_params = _params["bass"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/settings/adjustments/bass",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_sound_settings_adjustments_directivity(
        self,
        directivity: Annotated[Directivity, Field(..., description="the directivity")],
        **kwargs,
    ) -> None:  # noqa: E501
        """Update the sound settings adjustments for directivity  # noqa: E501


        :param directivity: the directivity (required)
        :type directivity: Directivity
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_sound_settings_adjustments_directivity_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_sound_settings_adjustments_directivity_with_http_info(
            directivity, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_sound_settings_adjustments_directivity_with_http_info(
        self,
        directivity: Annotated[Directivity, Field(..., description="the directivity")],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Update the sound settings adjustments for directivity  # noqa: E501


        :param directivity: the directivity (required)
        :type directivity: Directivity
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["directivity"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_sound_settings_adjustments_directivity" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["directivity"] is not None:
            _body_params = _params["directivity"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/settings/adjustments/directivity",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_sound_settings_adjustments_loudness(
        self,
        loudness: Annotated[Loudness, Field(..., description="the loudness mode")],
        **kwargs,
    ) -> None:  # noqa: E501
        """Update the sound settings adjustments for loudness  # noqa: E501


        :param loudness: the loudness mode (required)
        :type loudness: Loudness
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_sound_settings_adjustments_loudness_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_sound_settings_adjustments_loudness_with_http_info(
            loudness, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_sound_settings_adjustments_loudness_with_http_info(
        self,
        loudness: Annotated[Loudness, Field(..., description="the loudness mode")],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Update the sound settings adjustments for loudness  # noqa: E501


        :param loudness: the loudness mode (required)
        :type loudness: Loudness
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["loudness"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_sound_settings_adjustments_loudness" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["loudness"] is not None:
            _body_params = _params["loudness"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/settings/adjustments/loudness",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_sound_settings_adjustments_treble(
        self,
        treble: Annotated[Treble, Field(..., description="the treble level")],
        **kwargs,
    ) -> None:  # noqa: E501
        """Update the sound settings adjustments for treble  # noqa: E501


        :param treble: the treble level (required)
        :type treble: Treble
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_sound_settings_adjustments_treble_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_sound_settings_adjustments_treble_with_http_info(
            treble, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_sound_settings_adjustments_treble_with_http_info(
        self,
        treble: Annotated[Treble, Field(..., description="the treble level")],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Update the sound settings adjustments for treble  # noqa: E501


        :param treble: the treble level (required)
        :type treble: Treble
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["treble"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_sound_settings_adjustments_treble" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["treble"] is not None:
            _body_params = _params["treble"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/settings/adjustments/treble",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_volume_mute(
        self,
        volume_mute: Annotated[
            VolumeMute, Field(..., description="The mute state to apply")
        ],
        **kwargs,
    ) -> None:  # noqa: E501
        """Set the volume mute state  # noqa: E501


        :param volume_mute: The mute state to apply (required)
        :type volume_mute: VolumeMute
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_volume_mute_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_volume_mute_with_http_info(volume_mute, **kwargs)  # noqa: E501

    @validate_arguments
    async def set_volume_mute_with_http_info(
        self,
        volume_mute: Annotated[
            VolumeMute, Field(..., description="The mute state to apply")
        ],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Set the volume mute state  # noqa: E501


        :param volume_mute: The mute state to apply (required)
        :type volume_mute: VolumeMute
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["volume_mute"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_volume_mute" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["volume_mute"] is not None:
            _body_params = _params["volume_mute"]

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/volume/mute",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    async def set_volume_settings(
        self,
        volume_settings: Annotated[
            VolumeSettings, Field(..., description="The settings to apply")
        ],
        **kwargs,
    ) -> None:  # noqa: E501
        """Set the current volume settings for default and maximum volume level  # noqa: E501


        :param volume_settings: The settings to apply (required)
        :type volume_settings: VolumeSettings
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_volume_settings_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_volume_settings_with_http_info(volume_settings, **kwargs)  # noqa: E501

    @validate_arguments
    async def set_volume_settings_with_http_info(
        self,
        volume_settings: Annotated[
            VolumeSettings, Field(..., description="The settings to apply")
        ],
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Set the current volume settings for default and maximum volume level  # noqa: E501


        :param volume_settings: The settings to apply (required)
        :type volume_settings: VolumeSettings
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["volume_settings"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_volume_settings" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["volume_settings"] is not None:
            _body_params = _params["volume_settings"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/sound/volume/settings",
            "PUT",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, conint
except ImportError:
    from pydantic import BaseModel, Field, conint


class OverlayPlayRequestCommon(BaseModel):
    """
    OverlayPlayRequestCommon
    """

    volume_absolute: Optional[conint(strict=True, le=100, ge=0)] = Field(
        default=None,
        alias="volumeAbsolute",
        description="An optional absolute volume level at which to play the URI. If not provided, the URI will play at the currently configured volume level on the product. The level should be provided in volume steps [0, 100] ",
    )
    __properties = ["volumeAbsolute"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OverlayPlayRequestCommon:
        """Create an instance of OverlayPlayRequestCommon from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if volume_absolute (nullable) is None
        # and __fields_set__ contains the field
        if self.volume_absolute is None and "volume_absolute" in self.__fields_set__:
            _dict["volumeAbsolute"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OverlayPlayRequestCommon:
        """Create an instance of OverlayPlayRequestCommon from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OverlayPlayRequestCommon.parse_obj(obj)

        _obj = OverlayPlayRequestCommon.parse_obj(
            {"volume_absolute": obj.get("volumeAbsolute")}
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, conint
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, conint


class ListeningModeRef(BaseModel):
    """
    Reference to a listening mode  # noqa: E501
    """

    href: Optional[StrictStr] = Field(
        default=...,
        description="Host relative URI of the listening mode or `null` if there is no active listening mode. ",
    )
    id: Optional[conint(strict=True, ge=0)] = Field(
        default=...,
        description="ID of the active listening mode or `null` if there is no active listening mode. ",
    )
    __properties = ["href", "id"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ListeningModeRef:
        """Create an instance of ListeningModeRef from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict["href"] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict["id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ListeningModeRef:
        """Create an instance of ListeningModeRef from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ListeningModeRef.parse_obj(obj)

        _obj = ListeningModeRef.parse_obj(
            {"href": obj.get("href"), "id": obj.get("id")}
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import List, Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, conlist, constr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, conlist, constr, validator

from mozart_api.models.action import Action


class Scene(BaseModel):
    """
    Scene
    """

    action_list: conlist(Action) = Field(
        default=...,
        alias="actionList",
        description="An ordered list of Actions to run on the product",
    )
    client_context: Optional[constr(strict=True, max_length=4096)] = Field(
        default=None,
        alias="clientContext",
        description="An optional generic string property supplied from the client. If supplied it will overwrite any currently stored clientContext. If not supplied any stored clientContext will be left unchanged. ",
    )
    label: Optional[StrictStr] = None
    tags: Optional[conlist(StrictStr)] = Field(
        default=None,
        description="A list of user defined tags. This allows a client to create virtual lists",
    )
    classification: Optional[StrictStr] = Field(
        default=None, description="The classification of Scene"
    )
    __properties = ["actionList", "clientContext", "label", "tags", "classification"]

    @validator("classification")
    def classification_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "system",
            "userDefined",
        ):
            raise ValueError("must be one of enum values ('system', 'userDefined')")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Scene:
        """Create an instance of Scene from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in action_list (list)
        _items = []
        if self.action_list:
            for _item in self.action_list:
                if _item:
                    _items.append(_item.to_dict())
            _dict["actionList"] = _items
        # set to None if client_context (nullable) is None
        # and __fields_set__ contains the field
        if self.client_context is None and "client_context" in self.__fields_set__:
            _dict["clientContext"] = None

        # set to None if label (nullable) is None
        # and __fields_set__ contains the field
        if self.label is None and "label" in self.__fields_set__:
            _dict["label"] = None

        # set to None if classification (nullable) is None
        # and __fields_set__ contains the field
        if self.classification is None and "classification" in self.__fields_set__:
            _dict["classification"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Scene:
        """Create an instance of Scene from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Scene.parse_obj(obj)

        _obj = Scene.parse_obj(
            {
                "action_list": [
                    Action.from_dict(_item) for _item in obj.get("actionList")
                ]
                if obj.get("actionList") is not None
                else None,
                "client_context": obj.get("clientContext"),
                "label": obj.get("label"),
                "tags": obj.get("tags"),
                "classification": obj.get("classification"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field
except ImportError:
    from pydantic import BaseModel, Field

from mozart_api.models.ambience import Ambience
from mozart_api.models.balance import Balance
from mozart_api.models.bass_management import BassManagement
from mozart_api.models.compression import Compression
from mozart_api.models.directivity import Directivity
from mozart_api.models.fader import Fader
from mozart_api.models.room_compensation import RoomCompensation
from mozart_api.models.spatial_envelopment import SpatialEnvelopment
from mozart_api.models.spatial_height import SpatialHeight
from mozart_api.models.spatial_processing import SpatialProcessing
from mozart_api.models.spatial_surround import SpatialSurround
from mozart_api.models.spatial_width import SpatialWidth
from mozart_api.models.speech_enhance import SpeechEnhance
from mozart_api.models.tone_touch import ToneTouch


class ListeningModeFeatures(BaseModel):
    """
    Sound features to apply  # noqa: E501
    """

    ambience: Optional[Ambience] = None
    balance: Optional[Balance] = None
    bass_management: Optional[BassManagement] = Field(
        default=None, alias="bassManagement"
    )
    compression: Optional[Compression] = None
    directivity: Optional[Directivity] = None
    fader: Optional[Fader] = None
    room_compensation: Optional[RoomCompensation] = Field(
        default=None, alias="roomCompensation"
    )
    spatial_envelopment: Optional[SpatialEnvelopment] = Field(
        default=None, alias="spatialEnvelopment"
    )
    spatial_height: Optional[SpatialHeight] = Field(default=None, alias="spatialHeight")
    spatial_processing: Optional[SpatialProcessing] = Field(
        default=None, alias="spatialProcessing"
    )
    spatial_surround: Optional[SpatialSurround] = Field(
        default=None, alias="spatialSurround"
    )
    spatial_width: Optional[SpatialWidth] = Field(default=None, alias="spatialWidth")
    speech_enhance: Optional[SpeechEnhance] = Field(default=None, alias="speechEnhance")
    tone_touch: Optional[ToneTouch] = Field(default=None, alias="toneTouch")
    __properties = [
        "ambience",
        "balance",
        "bassManagement",
        "compression",
        "directivity",
        "fader",
        "roomCompensation",
        "spatialEnvelopment",
        "spatialHeight",
        "spatialProcessing",
        "spatialSurround",
        "spatialWidth",
        "speechEnhance",
        "toneTouch",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ListeningModeFeatures:
        """Create an instance of ListeningModeFeatures from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of ambience
        if self.ambience:
            _dict["ambience"] = self.ambience.to_dict()
        # override the default output from pydantic by calling `to_dict()` of balance
        if self.balance:
            _dict["balance"] = self.balance.to_dict()
        # override the default output from pydantic by calling `to_dict()` of bass_management
        if self.bass_management:
            _dict["bassManagement"] = self.bass_management.to_dict()
        # override the default output from pydantic by calling `to_dict()` of compression
        if self.compression:
            _dict["compression"] = self.compression.to_dict()
        # override the default output from pydantic by calling `to_dict()` of directivity
        if self.directivity:
            _dict["directivity"] = self.directivity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of fader
        if self.fader:
            _dict["fader"] = self.fader.to_dict()
        # override the default output from pydantic by calling `to_dict()` of room_compensation
        if self.room_compensation:
            _dict["roomCompensation"] = self.room_compensation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spatial_envelopment
        if self.spatial_envelopment:
            _dict["spatialEnvelopment"] = self.spatial_envelopment.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spatial_height
        if self.spatial_height:
            _dict["spatialHeight"] = self.spatial_height.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spatial_processing
        if self.spatial_processing:
            _dict["spatialProcessing"] = self.spatial_processing.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spatial_surround
        if self.spatial_surround:
            _dict["spatialSurround"] = self.spatial_surround.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spatial_width
        if self.spatial_width:
            _dict["spatialWidth"] = self.spatial_width.to_dict()
        # override the default output from pydantic by calling `to_dict()` of speech_enhance
        if self.speech_enhance:
            _dict["speechEnhance"] = self.speech_enhance.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tone_touch
        if self.tone_touch:
            _dict["toneTouch"] = self.tone_touch.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ListeningModeFeatures:
        """Create an instance of ListeningModeFeatures from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ListeningModeFeatures.parse_obj(obj)

        _obj = ListeningModeFeatures.parse_obj(
            {
                "ambience": Ambience.from_dict(obj.get("ambience"))
                if obj.get("ambience") is not None
                else None,
                "balance": Balance.from_dict(obj.get("balance"))
                if obj.get("balance") is not None
                else None,
                "bass_management": BassManagement.from_dict(obj.get("bassManagement"))
                if obj.get("bassManagement") is not None
                else None,
                "compression": Compression.from_dict(obj.get("compression"))
                if obj.get("compression") is not None
                else None,
                "directivity": Directivity.from_dict(obj.get("directivity"))
                if obj.get("directivity") is not None
                else None,
                "fader": Fader.from_dict(obj.get("fader"))
                if obj.get("fader") is not None
                else None,
                "room_compensation": RoomCompensation.from_dict(
                    obj.get("roomCompensation")
                )
                if obj.get("roomCompensation") is not None
                else None,
                "spatial_envelopment": SpatialEnvelopment.from_dict(
                    obj.get("spatialEnvelopment")
                )
                if obj.get("spatialEnvelopment") is not None
                else None,
                "spatial_height": SpatialHeight.from_dict(obj.get("spatialHeight"))
                if obj.get("spatialHeight") is not None
                else None,
                "spatial_processing": SpatialProcessing.from_dict(
                    obj.get("spatialProcessing")
                )
                if obj.get("spatialProcessing") is not None
                else None,
                "spatial_surround": SpatialSurround.from_dict(
                    obj.get("spatialSurround")
                )
                if obj.get("spatialSurround") is not None
                else None,
                "spatial_width": SpatialWidth.from_dict(obj.get("spatialWidth"))
                if obj.get("spatialWidth") is not None
                else None,
                "speech_enhance": SpeechEnhance.from_dict(obj.get("speechEnhance"))
                if obj.get("speechEnhance") is not None
                else None,
                "tone_touch": ToneTouch.from_dict(obj.get("toneTouch"))
                if obj.get("toneTouch") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, constr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, constr, validator


class SpeakerLinkMemberStatus(BaseModel):
    """
    SpeakerLinkMemberStatus
    """

    base_url: Optional[StrictStr] = Field(
        default=None,
        alias="baseUrl",
        description="API prefix to use when talking to the secondary",
    )
    health: StrictStr = Field(...)
    ip_address: Optional[StrictStr] = Field(
        default=None, alias="ipAddress", description="IP address"
    )
    network_type: StrictStr = Field(default=..., alias="networkType")
    product_type: Optional[StrictStr] = Field(default=None, alias="productType")
    serial_number: constr(strict=True) = Field(
        default=..., alias="serialNumber", description="Speaker serial number"
    )
    updated_at: datetime = Field(
        default=..., alias="updatedAt", description="Timestamp in RFC3393 format"
    )
    __properties = [
        "baseUrl",
        "health",
        "ipAddress",
        "networkType",
        "productType",
        "serialNumber",
        "updatedAt",
    ]

    @validator("health")
    def health_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "unknown",
            "good",
            "warning",
            "bad",
        ):
            raise ValueError(
                "must be one of enum values ('unknown', 'good', 'warning', 'bad')"
            )
        return value

    @validator("network_type")
    def network_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "none",
            "wired",
            "wireless",
        ):
            raise ValueError("must be one of enum values ('none', 'wired', 'wireless')")
        return value

    @validator("serial_number")
    def serial_number_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^\d{8}", value):
            raise ValueError(r"must validate the regular expression /^\d{8}/")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SpeakerLinkMemberStatus:
        """Create an instance of SpeakerLinkMemberStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if base_url (nullable) is None
        # and __fields_set__ contains the field
        if self.base_url is None and "base_url" in self.__fields_set__:
            _dict["baseUrl"] = None

        # set to None if ip_address (nullable) is None
        # and __fields_set__ contains the field
        if self.ip_address is None and "ip_address" in self.__fields_set__:
            _dict["ipAddress"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SpeakerLinkMemberStatus:
        """Create an instance of SpeakerLinkMemberStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SpeakerLinkMemberStatus.parse_obj(obj)

        _obj = SpeakerLinkMemberStatus.parse_obj(
            {
                "base_url": obj.get("baseUrl"),
                "health": obj.get("health"),
                "ip_address": obj.get("ipAddress"),
                "network_type": obj.get("networkType"),
                "product_type": obj.get("productType"),
                "serial_number": obj.get("serialNumber"),
                "updated_at": obj.get("updatedAt"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictInt, StrictStr, validator

from mozart_api.models.play_queue_item_type import PlayQueueItemType
from mozart_api.models.playback_content_metadata import PlaybackContentMetadata


class PlayQueueItem(BaseModel):
    """
    PlayQueueItem
    """

    index: Optional[StrictInt] = None
    metadata: Optional[PlaybackContentMetadata] = None
    playlist_sort_by: Optional[StrictStr] = Field(
        default=None,
        alias="playlistSortBy",
        description="Option to choose the order that a playlist is sorted by",
    )
    provider: PlayQueueItemType = Field(...)
    start_now_from_position: Optional[StrictInt] = Field(
        default=None,
        alias="startNowFromPosition",
        description="Start playing specified track as soon as possible. This works for any type, for `track` set to 0 and for `playlist` specify the wanted track from position 0",
    )
    type: StrictStr = Field(...)
    uri: StrictStr = Field(
        default=...,
        description="This can be url, track id, playlist id, radio id and suchlike",
    )
    __properties = [
        "index",
        "metadata",
        "playlistSortBy",
        "provider",
        "startNowFromPosition",
        "type",
        "uri",
    ]

    @validator("playlist_sort_by")
    def playlist_sort_by_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "name",
            "artist",
            "date",
            "album",
        ):
            raise ValueError(
                "must be one of enum values ('name', 'artist', 'date', 'album')"
            )
        return value

    @validator("type")
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "track",
            "playlist",
        ):
            raise ValueError("must be one of enum values ('track', 'playlist')")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PlayQueueItem:
        """Create an instance of PlayQueueItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict["metadata"] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of provider
        if self.provider:
            _dict["provider"] = self.provider.to_dict()
        # set to None if playlist_sort_by (nullable) is None
        # and __fields_set__ contains the field
        if self.playlist_sort_by is None and "playlist_sort_by" in self.__fields_set__:
            _dict["playlistSortBy"] = None

        # set to None if start_now_from_position (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.start_now_from_position is None
            and "start_now_from_position" in self.__fields_set__
        ):
            _dict["startNowFromPosition"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PlayQueueItem:
        """Create an instance of PlayQueueItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PlayQueueItem.parse_obj(obj)

        _obj = PlayQueueItem.parse_obj(
            {
                "index": obj.get("index"),
                "metadata": PlaybackContentMetadata.from_dict(obj.get("metadata"))
                if obj.get("metadata") is not None
                else None,
                "playlist_sort_by": obj.get("playlistSortBy"),
                "provider": PlayQueueItemType.from_dict(obj.get("provider"))
                if obj.get("provider") is not None
                else None,
                "start_now_from_position": obj.get("startNowFromPosition"),
                "type": obj.get("type"),
                "uri": obj.get("uri"),
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictInt
except ImportError:
    from pydantic import BaseModel, Field, StrictInt


class PowerLinkTrigger(BaseModel):
    """
    PowerLinkTrigger
    """

    power_link_preset: Optional[StrictInt] = Field(
        default=None,
        alias="powerLinkPreset",
        description="Despite the name, this preset is used anytime a speaker is used as a secondary as a way for the primary to indicate which directivity (and possibly other caracteristics in the future) should be used on a speaker by speaker basis. It is used when a speaker is a powerlink speaker (hence the name), WISA speaker or a speakerlink secondary in multichannel. ",
    )
    __properties = ["powerLinkPreset"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PowerLinkTrigger:
        """Create an instance of PowerLinkTrigger from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PowerLinkTrigger:
        """Create an instance of PowerLinkTrigger from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PowerLinkTrigger.parse_obj(obj)

        _obj = PowerLinkTrigger.parse_obj(
            {"power_link_preset": obj.get("powerLinkPreset")}
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, StrictStr, validator
except ImportError:
    from pydantic import BaseModel, StrictStr, validator


class PowerStateEnum(BaseModel):
    """
    PowerStateEnum
    """

    value: Optional[StrictStr] = None
    __properties = ["value"]

    @validator("value")
    def value_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in (
            "networkStandby",
            "on",
            "standby",
            "shutdown",
            "storage",
        ):
            raise ValueError(
                "must be one of enum values ('networkStandby', 'on', 'standby', 'shutdown', 'storage')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PowerStateEnum:
        """Create an instance of PowerStateEnum from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PowerStateEnum:
        """Create an instance of PowerStateEnum from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PowerStateEnum.parse_obj(obj)

        _obj = PowerStateEnum.parse_obj({"value": obj.get("value")})
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, constr
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, constr


class OverlayPlayRequestTextToSpeechTextToSpeech(BaseModel):
    """
    Object used for performing a text-to-speech operation on the product.   # noqa: E501
    """

    lang: Optional[StrictStr] = Field(
        default=None,
        description="An optional specification of the language to use. The default value is ultimately decided by the BeoCloud API, but currently it defaults to `en-us`. ",
    )
    text: constr(strict=True, max_length=2048) = Field(
        default=...,
        description="Required field containing the actual text that will be converted to speech (audio). A maximum length of 2048 characters is allowed. ",
    )
    __properties = ["lang", "text"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OverlayPlayRequestTextToSpeechTextToSpeech:
        """Create an instance of OverlayPlayRequestTextToSpeechTextToSpeech from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if lang (nullable) is None
        # and __fields_set__ contains the field
        if self.lang is None and "lang" in self.__fields_set__:
            _dict["lang"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OverlayPlayRequestTextToSpeechTextToSpeech:
        """Create an instance of OverlayPlayRequestTextToSpeechTextToSpeech from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OverlayPlayRequestTextToSpeechTextToSpeech.parse_obj(obj)

        _obj = OverlayPlayRequestTextToSpeechTextToSpeech.parse_obj(
            {"lang": obj.get("lang"), "text": obj.get("text")}
        )
        return _obj



"""Helper classes for using auto-generated API."""

import asyncio
import contextlib
import json
import logging
import re
from collections import defaultdict
from collections.abc import Awaitable, Callable
from contextlib import AbstractAsyncContextManager
from dataclasses import dataclass
from datetime import time
from ssl import SSLContext
from types import TracebackType
from typing import Literal, TypedDict

from aiohttp import ClientSession, ClientTimeout
from aiohttp.client_exceptions import (
    ClientConnectorError,
    ClientOSError,
    ServerTimeoutError,
)
from inflection import underscore

from mozart_api.api.mozart_api import MozartApi
from mozart_api.api_client import ApiClient
from mozart_api.configuration import Configuration
from mozart_api.exceptions import ApiException

# Generated section start
from mozart_api.models import (
    AlarmTimerEventData,
    AlarmTriggeredInfo,
    Art,
    BatteryState,
    BeolinkExperiencesResult,
    BeolinkJoinResult,
    BeoRemoteButton,
    ButtonEvent,
    HdmiInput,
    HdmiVideoFormat,
    InstallRecordIdState,
    ListeningModeProps,
    PlaybackContentMetadata,
    PlaybackError,
    PlaybackProgress,
    PowerlinkConnectionStateEnum,
    PowerStateEnum,
    ProductCurtainStatus,
    RenderingState,
    RoomCompensationCurrentMeasurement,
    RoomCompensationStateValue,
    SoftwareUpdateState,
    SoundSettings,
    Source,
    SpeakerGroupOverview,
    SpeakerLinkStatus,
    SpeakerRoleEnum,
    StandConnected,
    StandPosition,
    TvInfoEventData,
    VolumeState,
    WebSocketEventActiveHdmiInputSignal,
    WebSocketEventActiveListeningMode,
    WebSocketEventActiveSpeakerGroup,
    WebSocketEventAlarmTimer,
    WebSocketEventAlarmTriggered,
    WebSocketEventBattery,
    WebSocketEventBeolinkExperiencesResult,
    WebSocketEventBeolinkJoinResult,
    WebSocketEventBeoRemoteButton,
    WebSocketEventButton,
    WebSocketEventCurtains,
    WebSocketEventHdmiVideoFormatSignal,
    WebSocketEventNotification,
    WebSocketEventPlaybackError,
    WebSocketEventPlaybackMetadata,
    WebSocketEventPlaybackProgress,
    WebSocketEventPlaybackSource,
    WebSocketEventPlaybackState,
    WebSocketEventPowerlinkConnectionState,
    WebSocketEventPowerState,
    WebSocketEventPucInstallRemoteIdStatus,
    WebSocketEventRole,
    WebSocketEventRoomCompensationCurrentMeasurementEvent,
    WebSocketEventRoomCompensationState,
    WebSocketEventSoftwareUpdateState,
    WebSocketEventSoundSettings,
    WebSocketEventSourceChange,
    WebSocketEventSpeakerGroupChanged,
    WebSocketEventSpeakerLinkStatusChanged,
    WebSocketEventStandConnected,
    WebSocketEventStandPosition,
    WebSocketEventTvInfo,
    WebSocketEventVolume,
    WebSocketEventWisaOutState,
    WebsocketNotificationTag,
    WisaOutState,
)

# Generated section end
from mozart_api.rest import RESTClientObject

# Generated section start

NOTIFICATION_TYPES = {
    "active_hdmi_input_signal",
    "active_listening_mode",
    "active_speaker_group",
    "alarm_timer",
    "alarm_triggered",
    "battery",
    "beo_remote_button",
    "beolink_experiences_result",
    "beolink_join_result",
    "button",
    "curtains",
    "hdmi_video_format_signal",
    "notification",
    "playback_error",
    "playback_metadata",
    "playback_progress",
    "playback_source",
    "playback_state",
    "power_state",
    "powerlink_connection_state",
    "puc_install_remote_id_status",
    "role",
    "room_compensation_current_measurement_event",
    "room_compensation_state",
    "software_update_state",
    "sound_settings",
    "source_change",
    "speaker_group_changed",
    "speaker_link_status_changed",
    "stand_connected",
    "stand_position",
    "tv_info",
    "volume",
    "wisa_out_state",
}

WebSocketEventType = type[
    WebSocketEventActiveHdmiInputSignal
    | WebSocketEventActiveListeningMode
    | WebSocketEventActiveSpeakerGroup
    | WebSocketEventAlarmTimer
    | WebSocketEventAlarmTriggered
    | WebSocketEventBattery
    | WebSocketEventBeoRemoteButton
    | WebSocketEventBeolinkExperiencesResult
    | WebSocketEventBeolinkJoinResult
    | WebSocketEventButton
    | WebSocketEventCurtains
    | WebSocketEventHdmiVideoFormatSignal
    | WebSocketEventNotification
    | WebSocketEventPlaybackError
    | WebSocketEventPlaybackMetadata
    | WebSocketEventPlaybackProgress
    | WebSocketEventPlaybackSource
    | WebSocketEventPlaybackState
    | WebSocketEventPowerState
    | WebSocketEventPowerlinkConnectionState
    | WebSocketEventPucInstallRemoteIdStatus
    | WebSocketEventRole
    | WebSocketEventRoomCompensationCurrentMeasurementEvent
    | WebSocketEventRoomCompensationState
    | WebSocketEventSoftwareUpdateState
    | WebSocketEventSoundSettings
    | WebSocketEventSourceChange
    | WebSocketEventSpeakerGroupChanged
    | WebSocketEventSpeakerLinkStatusChanged
    | WebSocketEventStandConnected
    | WebSocketEventStandPosition
    | WebSocketEventTvInfo
    | WebSocketEventVolume
    | WebSocketEventWisaOutState
]

# Generated section end

WEBSOCKET_TIMEOUT = 5.0

logger = logging.getLogger(__name__)


def refactor_name(notification_type: str) -> str:
    """Remove WebSocketEvent prefix from string and convert to snake_case."""
    return underscore(notification_type.removeprefix("WebSocketEvent"))


def time_to_seconds(time_object: time) -> int:
    """Convert time object to number of seconds."""
    return (time_object.hour * 60 * 60) + (time_object.minute * 60) + time_object.second


def check_valid_jid(jid: str) -> bool:
    """Check if a JID is valid."""
    pattern = re.compile(r"(^\d{4})[.](\d{7})[.](\d{8})(@products\.bang-olufsen\.com)$")

    return pattern.fullmatch(jid) is not None


def check_valid_serial_number(serial_number: str) -> bool:
    """Check if a serial_number is valid."""
    return bool(re.fullmatch(r"\d{8}", serial_number))


def get_highest_resolution_artwork(metadata: PlaybackContentMetadata) -> Art:
    """Get the highest resolution Art from provided PlaybackContentMetadata."""
    # Return an empty Art if no artwork is in metadata to ensure no stale artwork
    if not metadata.art:
        return Art()

    # Dict for sorting images that have size defined by a string
    art_size = {"small": 1, "medium": 2, "large": 3}

    images = []

    # Images either have a key for specifying resolution or a "size" for the image.
    for image in metadata.art:
        # Skip any invalid artwork
        if not image.url:
            continue
        # Netradio.
        if image.key:
            images.append(int(image.key.split("x")[0]))
        # Everything else.
        elif image.size:
            images.append(art_size[image.size])

    # Check if only invalid images were provided
    if not images:
        return Art()

    # Choose the largest image.
    return metadata.art[images.index(max(images))]


class BaseWebSocketResponse(TypedDict):
    """Base class for serialized WebSocket notifications."""

    eventType: str
    eventDate: dict


class MozartClient(MozartApi):
    """User friendly Mozart REST API and WebSocket client."""

    def __init__(self, host: str, ssl_context: SSLContext | None = None) -> None:
        """Initialize Mozart client."""
        self.host = host
        self.websocket_connected = False
        self.websocket_reconnect = False

        self._websocket_listener_active = False
        self._websocket_listener_remote_listener_active = False
        self._websocket_listeners_active = False
        self._websocket_tasks: set[asyncio.Task] = set()

        self._on_connection_lost: Callable | None = None
        self._on_connection: Callable | None = None

        self._on_all_notifications: (
            Callable[[WebSocketEventType, str], Awaitable[None] | None] | None
        ) = None
        self._on_all_notifications_raw: (
            Callable[[BaseWebSocketResponse], Awaitable[None] | None] | None
        ) = None

        self._notification_callbacks: dict[str, Callable | None] = defaultdict()
        self._notification_callbacks.default_factory = lambda: None

        # Configure MozartApi object.
        self.configuration = Configuration(host=f"http://{self.host}")
        self.configuration.verify_ssl = False

        if ssl_context is None:
            super().__init__(ApiClient(self.configuration))
        elif ssl_context:
            super().__init__(
                ApiClient(
                    self.configuration,
                    rest_client=RESTClientObject(
                        configuration=self.configuration,
                        ssl_context=ssl_context,
                    ),
                ),
            )

    async def close_api_client(self) -> None:
        """Close the API ClientSession."""
        await self.api_client.close()

    async def __aenter__(self) -> AbstractAsyncContextManager:
        """Context entry."""
        if self.api_client.rest_client.pool_manager.closed:
            self.api_client = ApiClient(self.configuration)
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None:
        """Context exit."""
        await self.close_api_client()

    async def _check_websocket_connection(
        self,
    ) -> Literal[True] | ClientConnectorError | ClientOSError | ServerTimeoutError:
        """Try to connect to the device's WebSocket notification channel."""
        try:
            async with (
                ClientSession(
                    timeout=ClientTimeout(total=WEBSOCKET_TIMEOUT),
                ) as session,
                session.ws_connect(
                    f"ws://{self.host}:9339/",
                    timeout=WEBSOCKET_TIMEOUT,
                    receive_timeout=WEBSOCKET_TIMEOUT,
                ) as websocket,
            ):
                if await websocket.receive():
                    return True

        except (ClientConnectorError, ClientOSError, ServerTimeoutError) as error:
            return error

    async def _check_api_connection(
        self,
    ) -> Literal[True] | ApiException | ClientConnectorError | TimeoutError:
        """Check if a connection can be made to the device's REST API."""
        try:
            await self.get_battery_state(_request_timeout=3)
        except (ApiException, ClientConnectorError, TimeoutError) as error:
            return error

        return True

    async def check_device_connection(self, raise_error: bool = False) -> bool:
        """Check API and WebSocket connection."""
        # Don't use a taskgroup as both tasks should always be checked
        tasks: tuple[
            asyncio.Task[
                Literal[True]
                | ClientConnectorError
                | ClientOSError
                | ServerTimeoutError
            ],
            asyncio.Task[
                Literal[True] | ApiException | ClientConnectorError | TimeoutError
            ],
        ] = (
            asyncio.create_task(self._check_websocket_connection(), name="websocket"),
            asyncio.create_task(self._check_api_connection(), name="REST API"),
        )

        # Wait for tasks to complete
        for task in tasks:
            while not task.done():
                await asyncio.sleep(0)

        errors: list[
            ClientConnectorError
            | ClientOSError
            | ServerTimeoutError
            | ApiException
            | TimeoutError
        ] = [result for task in tasks if (result := task.result()) is not True]

        connection_available = bool(len(errors) == 0)

        # Raise any exceptions
        if not connection_available and raise_error:
            msg = f"Can't connect to {self.host}"
            raise ExceptionGroup(msg, tuple(errors))

        return connection_available

    async def connect_notifications(
        self,
        remote_control: bool = False,
        reconnect: bool = False,
    ) -> None:
        """Start the WebSocket listener task(s)."""
        self.websocket_reconnect = reconnect

        # Always add main WebSocket listener
        if not self._websocket_listener_active:
            self._websocket_tasks.add(
                asyncio.create_task(
                    coro=self._websocket_listener(f"ws://{self.host}:9339/"),
                    name=f"{self.host} - listener task",
                ),
            )

            self._websocket_listener_active = True

        # Add WebSocket listener for remote control events if defined.
        if not self._websocket_listener_remote_listener_active and remote_control:
            self._websocket_tasks.add(
                asyncio.create_task(
                    coro=self._websocket_listener(
                        f"ws://{self.host}:9339/remoteControl",
                    ),
                    name=f"{self.host} - remote listener task",
                ),
            )
            self._websocket_listener_remote_listener_active = True

        self._websocket_listeners_active = True

    def disconnect_notifications(self) -> None:
        """Stop the WebSocket listener tasks."""
        self._websocket_listener_active = False
        self._websocket_listener_remote_listener_active = False
        self._websocket_listeners_active = False

        for task in self._websocket_tasks:
            task.cancel()

        self._websocket_tasks.clear()

    async def _websocket_listener(self, host: str) -> None:
        """WebSocket listener."""
        while True:
            try:
                async with (
                    ClientSession(
                        timeout=ClientTimeout(total=WEBSOCKET_TIMEOUT),
                    ) as session,
                    session.ws_connect(
                        url=host,
                        heartbeat=WEBSOCKET_TIMEOUT,
                    ) as websocket,
                ):
                    self.websocket_connected = True

                    if self._on_connection:
                        await self._trigger_callback(self._on_connection)

                    while self._websocket_listeners_active:
                        with contextlib.suppress(asyncio.TimeoutError):
                            # Receive JSON in order to get the
                            # Websocket notification name for deserialization
                            notification = await asyncio.wait_for(
                                websocket.receive_json(),
                                timeout=WEBSOCKET_TIMEOUT,
                            )

                            # Ensure that any notifications received after the
                            # disconnect command has been executed are not processed
                            if not self._websocket_listeners_active:
                                break

                            await self._on_message(notification)

                    # Disconnect
                    self.websocket_connected = False
                    await websocket.close()
                    return

            except (
                ClientConnectorError,
                ClientOSError,
                TypeError,
                ServerTimeoutError,
            ) as error:
                if self.websocket_connected:
                    logger.debug("%s : %s - %s", host, type(error), error)
                    self.websocket_connected = False

                    if self._on_connection_lost:
                        await self._trigger_callback(self._on_connection_lost)

                if not self.websocket_reconnect:
                    logger.exception("%s", host)
                    self.disconnect_notifications()
                    return

                await asyncio.sleep(WEBSOCKET_TIMEOUT)

    @dataclass
    class _ResponseWrapper:
        """Wrapper class for deserializing WebSocket response."""

        data: str

    async def _on_message(self, notification: BaseWebSocketResponse) -> None:
        """Handle WebSocket notifications."""
        # Get the object type and deserialized object.
        try:
            notification_type = notification["eventType"]

            deserialized_data = self.api_client.deserialize(
                self._ResponseWrapper(json.dumps(notification)),
                notification_type,
            ).event_data

        except (ValueError, AttributeError):
            logger.exception(
                "%s unable to deserialize WebSocket notification: (%s : %s)",
                self.host,
                notification_type,
                notification,
            )
            return

        # Ensure that only valid notifications trigger callbacks
        if deserialized_data is None:
            return

        # Handle all notifications if defined
        if self._on_all_notifications:
            await self._trigger_callback(
                self._on_all_notifications,
                notification,
                refactor_name(notification_type),
            )

        if self._on_all_notifications_raw:
            await self._trigger_callback(self._on_all_notifications_raw, notification)

        # Handle specific notifications if defined
        triggered_notification = self._notification_callbacks[notification_type]

        if triggered_notification:
            await self._trigger_callback(triggered_notification, deserialized_data)

    async def _trigger_callback(
        self,
        callback: Callable,
        *args: BaseWebSocketResponse | dict | str | WebSocketEventType,
    ) -> None:
        """Trigger async or sync callback correctly."""
        if asyncio.iscoroutinefunction(callback):
            await callback(*args)
        else:
            callback(*args)

    def get_on_connection_lost(self, on_connection_lost: Callable) -> None:
        """Set callback for WebSocket connection lost."""
        self._on_connection_lost = on_connection_lost

    def get_on_connection(self, on_connection: Callable) -> None:
        """Set callback for WebSocket connection."""
        self._on_connection = on_connection

    def get_all_notifications(
        self,
        on_all_notifications: Callable[
            [WebSocketEventType, str], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for all notifications."""
        self._on_all_notifications = on_all_notifications

    def get_all_notifications_raw(
        self,
        on_all_notifications_raw: Callable[
            [BaseWebSocketResponse], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for all notifications as dict."""
        self._on_all_notifications_raw = on_all_notifications_raw

    # Generated section start

    def get_active_hdmi_input_signal_notifications(
        self,
        on_active_hdmi_input_signal_notification: Callable[
            [HdmiInput], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventActiveHdmiInputSignal notifications."""
        self._notification_callbacks["WebSocketEventActiveHdmiInputSignal"] = (
            on_active_hdmi_input_signal_notification
        )

    def get_active_listening_mode_notifications(
        self,
        on_active_listening_mode_notification: Callable[
            [ListeningModeProps], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventActiveListeningMode notifications."""
        self._notification_callbacks["WebSocketEventActiveListeningMode"] = (
            on_active_listening_mode_notification
        )

    def get_active_speaker_group_notifications(
        self,
        on_active_speaker_group_notification: Callable[
            [SpeakerGroupOverview], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventActiveSpeakerGroup notifications."""
        self._notification_callbacks["WebSocketEventActiveSpeakerGroup"] = (
            on_active_speaker_group_notification
        )

    def get_alarm_timer_notifications(
        self,
        on_alarm_timer_notification: Callable[
            [AlarmTimerEventData], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventAlarmTimer notifications."""
        self._notification_callbacks["WebSocketEventAlarmTimer"] = (
            on_alarm_timer_notification
        )

    def get_alarm_triggered_notifications(
        self,
        on_alarm_triggered_notification: Callable[
            [AlarmTriggeredInfo], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventAlarmTriggered notifications."""
        self._notification_callbacks["WebSocketEventAlarmTriggered"] = (
            on_alarm_triggered_notification
        )

    def get_battery_notifications(
        self, on_battery_notification: Callable[[BatteryState], Awaitable[None] | None]
    ) -> None:
        """Set callback for WebSocketEventBattery notifications."""
        self._notification_callbacks["WebSocketEventBattery"] = on_battery_notification

    def get_beo_remote_button_notifications(
        self,
        on_beo_remote_button_notification: Callable[
            [BeoRemoteButton], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventBeoRemoteButton notifications."""
        self._notification_callbacks["WebSocketEventBeoRemoteButton"] = (
            on_beo_remote_button_notification
        )

    def get_beolink_experiences_result_notifications(
        self,
        on_beolink_experiences_result_notification: Callable[
            [BeolinkExperiencesResult], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventBeolinkExperiencesResult notifications."""
        self._notification_callbacks["WebSocketEventBeolinkExperiencesResult"] = (
            on_beolink_experiences_result_notification
        )

    def get_beolink_join_result_notifications(
        self,
        on_beolink_join_result_notification: Callable[
            [BeolinkJoinResult], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventBeolinkJoinResult notifications."""
        self._notification_callbacks["WebSocketEventBeolinkJoinResult"] = (
            on_beolink_join_result_notification
        )

    def get_button_notifications(
        self, on_button_notification: Callable[[ButtonEvent], Awaitable[None] | None]
    ) -> None:
        """Set callback for WebSocketEventButton notifications."""
        self._notification_callbacks["WebSocketEventButton"] = on_button_notification

    def get_curtains_notifications(
        self,
        on_curtains_notification: Callable[
            [ProductCurtainStatus], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventCurtains notifications."""
        self._notification_callbacks["WebSocketEventCurtains"] = (
            on_curtains_notification
        )

    def get_hdmi_video_format_signal_notifications(
        self,
        on_hdmi_video_format_signal_notification: Callable[
            [HdmiVideoFormat], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventHdmiVideoFormatSignal notifications."""
        self._notification_callbacks["WebSocketEventHdmiVideoFormatSignal"] = (
            on_hdmi_video_format_signal_notification
        )

    def get_notification_notifications(
        self,
        on_notification_notification: Callable[
            [WebsocketNotificationTag], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventNotification notifications."""
        self._notification_callbacks["WebSocketEventNotification"] = (
            on_notification_notification
        )

    def get_playback_error_notifications(
        self,
        on_playback_error_notification: Callable[
            [PlaybackError], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPlaybackError notifications."""
        self._notification_callbacks["WebSocketEventPlaybackError"] = (
            on_playback_error_notification
        )

    def get_playback_metadata_notifications(
        self,
        on_playback_metadata_notification: Callable[
            [PlaybackContentMetadata], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPlaybackMetadata notifications."""
        self._notification_callbacks["WebSocketEventPlaybackMetadata"] = (
            on_playback_metadata_notification
        )

    def get_playback_progress_notifications(
        self,
        on_playback_progress_notification: Callable[
            [PlaybackProgress], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPlaybackProgress notifications."""
        self._notification_callbacks["WebSocketEventPlaybackProgress"] = (
            on_playback_progress_notification
        )

    def get_playback_source_notifications(
        self,
        on_playback_source_notification: Callable[[Source], Awaitable[None] | None],
    ) -> None:
        """Set callback for WebSocketEventPlaybackSource notifications."""
        self._notification_callbacks["WebSocketEventPlaybackSource"] = (
            on_playback_source_notification
        )

    def get_playback_state_notifications(
        self,
        on_playback_state_notification: Callable[
            [RenderingState], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPlaybackState notifications."""
        self._notification_callbacks["WebSocketEventPlaybackState"] = (
            on_playback_state_notification
        )

    def get_power_state_notifications(
        self,
        on_power_state_notification: Callable[[PowerStateEnum], Awaitable[None] | None],
    ) -> None:
        """Set callback for WebSocketEventPowerState notifications."""
        self._notification_callbacks["WebSocketEventPowerState"] = (
            on_power_state_notification
        )

    def get_powerlink_connection_state_notifications(
        self,
        on_powerlink_connection_state_notification: Callable[
            [PowerlinkConnectionStateEnum], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPowerlinkConnectionState notifications."""
        self._notification_callbacks["WebSocketEventPowerlinkConnectionState"] = (
            on_powerlink_connection_state_notification
        )

    def get_puc_install_remote_id_status_notifications(
        self,
        on_puc_install_remote_id_status_notification: Callable[
            [InstallRecordIdState], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventPucInstallRemoteIdStatus notifications."""
        self._notification_callbacks["WebSocketEventPucInstallRemoteIdStatus"] = (
            on_puc_install_remote_id_status_notification
        )

    def get_role_notifications(
        self, on_role_notification: Callable[[SpeakerRoleEnum], Awaitable[None] | None]
    ) -> None:
        """Set callback for WebSocketEventRole notifications."""
        self._notification_callbacks["WebSocketEventRole"] = on_role_notification

    def get_room_compensation_current_measurement_event_notifications(
        self,
        on_room_compensation_current_measurement_event_notification: Callable[
            [RoomCompensationCurrentMeasurement], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventRoomCompensationCurrentMeasurementEvent notifications."""
        self._notification_callbacks[
            "WebSocketEventRoomCompensationCurrentMeasurementEvent"
        ] = on_room_compensation_current_measurement_event_notification

    def get_room_compensation_state_notifications(
        self,
        on_room_compensation_state_notification: Callable[
            [RoomCompensationStateValue], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventRoomCompensationState notifications."""
        self._notification_callbacks["WebSocketEventRoomCompensationState"] = (
            on_room_compensation_state_notification
        )

    def get_software_update_state_notifications(
        self,
        on_software_update_state_notification: Callable[
            [SoftwareUpdateState], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventSoftwareUpdateState notifications."""
        self._notification_callbacks["WebSocketEventSoftwareUpdateState"] = (
            on_software_update_state_notification
        )

    def get_sound_settings_notifications(
        self,
        on_sound_settings_notification: Callable[
            [SoundSettings], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventSoundSettings notifications."""
        self._notification_callbacks["WebSocketEventSoundSettings"] = (
            on_sound_settings_notification
        )

    def get_source_change_notifications(
        self, on_source_change_notification: Callable[[Source], Awaitable[None] | None]
    ) -> None:
        """Set callback for WebSocketEventSourceChange notifications."""
        self._notification_callbacks["WebSocketEventSourceChange"] = (
            on_source_change_notification
        )

    def get_speaker_group_changed_notifications(
        self,
        on_speaker_group_changed_notification: Callable[[str], Awaitable[None] | None],
    ) -> None:
        """Set callback for WebSocketEventSpeakerGroupChanged notifications."""
        self._notification_callbacks["WebSocketEventSpeakerGroupChanged"] = (
            on_speaker_group_changed_notification
        )

    def get_speaker_link_status_changed_notifications(
        self,
        on_speaker_link_status_changed_notification: Callable[
            [SpeakerLinkStatus], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventSpeakerLinkStatusChanged notifications."""
        self._notification_callbacks["WebSocketEventSpeakerLinkStatusChanged"] = (
            on_speaker_link_status_changed_notification
        )

    def get_stand_connected_notifications(
        self,
        on_stand_connected_notification: Callable[
            [StandConnected], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventStandConnected notifications."""
        self._notification_callbacks["WebSocketEventStandConnected"] = (
            on_stand_connected_notification
        )

    def get_stand_position_notifications(
        self,
        on_stand_position_notification: Callable[
            [StandPosition], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventStandPosition notifications."""
        self._notification_callbacks["WebSocketEventStandPosition"] = (
            on_stand_position_notification
        )

    def get_tv_info_notifications(
        self,
        on_tv_info_notification: Callable[[TvInfoEventData], Awaitable[None] | None],
    ) -> None:
        """Set callback for WebSocketEventTvInfo notifications."""
        self._notification_callbacks["WebSocketEventTvInfo"] = on_tv_info_notification

    def get_volume_notifications(
        self, on_volume_notification: Callable[[VolumeState], Awaitable[None] | None]
    ) -> None:
        """Set callback for WebSocketEventVolume notifications."""
        self._notification_callbacks["WebSocketEventVolume"] = on_volume_notification

    def get_wisa_out_state_notifications(
        self,
        on_wisa_out_state_notification: Callable[
            [WisaOutState], Awaitable[None] | None
        ],
    ) -> None:
        """Set callback for WebSocketEventWisaOutState notifications."""
        self._notification_callbacks["WebSocketEventWisaOutState"] = (
            on_wisa_out_state_notification
        )

    # Generated section end



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field
except ImportError:
    from pydantic import BaseModel, Field

from mozart_api.models.tv_integration_types import TvIntegrationTypes
from mozart_api.models.tv_properties import TvProperties
from mozart_api.models.tv_sound_settings import TvSoundSettings


class TvState(BaseModel):
    """
    TvState
    """

    integration: Optional[TvIntegrationTypes] = None
    sound_settings: Optional[TvSoundSettings] = Field(
        default=None, alias="soundSettings"
    )
    tv_info: Optional[TvProperties] = Field(default=None, alias="tvInfo")
    __properties = ["integration", "soundSettings", "tvInfo"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TvState:
        """Create an instance of TvState from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of integration
        if self.integration:
            _dict["integration"] = self.integration.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sound_settings
        if self.sound_settings:
            _dict["soundSettings"] = self.sound_settings.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tv_info
        if self.tv_info:
            _dict["tvInfo"] = self.tv_info.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TvState:
        """Create an instance of TvState from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TvState.parse_obj(obj)

        _obj = TvState.parse_obj(
            {
                "integration": TvIntegrationTypes.from_dict(obj.get("integration"))
                if obj.get("integration") is not None
                else None,
                "sound_settings": TvSoundSettings.from_dict(obj.get("soundSettings"))
                if obj.get("soundSettings") is not None
                else None,
                "tv_info": TvProperties.from_dict(obj.get("tvInfo"))
                if obj.get("tvInfo") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import io
import re  # noqa: F401
import warnings

try:
    from pydantic.v1 import ValidationError, validate_arguments
except ImportError:
    from pydantic import ValidationError, validate_arguments

from typing import Awaitable, Optional, Union, overload

from typing_extensions import Annotated

try:
    from pydantic.v1 import Field, StrictBool
except ImportError:
    from pydantic import Field, StrictBool


from typing import Optional

from mozart_api.api_client import ApiClient
from mozart_api.api_response import ApiResponse
from mozart_api.exceptions import ApiTypeError, ApiValueError  # noqa: F401
from mozart_api.models.stand_movement import StandMovement


class StandApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    async def set_stand_movement(
        self,
        stand_movement: Annotated[
            StandMovement, Field(..., description="Stand Movement")
        ],
        slow_speed: Annotated[
            Optional[StrictBool],
            Field(description="Use slow speed on continuous movement"),
        ] = None,
        **kwargs,
    ) -> None:  # noqa: E501
        """Set stand movement with either an angle or a movement type  # noqa: E501


        :param stand_movement: Stand Movement (required)
        :type stand_movement: StandMovement
        :param slow_speed: Use slow speed on continuous movement
        :type slow_speed: bool
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_stand_movement_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_stand_movement_with_http_info(
            stand_movement, slow_speed, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_stand_movement_with_http_info(
        self,
        stand_movement: Annotated[
            StandMovement, Field(..., description="Stand Movement")
        ],
        slow_speed: Annotated[
            Optional[StrictBool],
            Field(description="Use slow speed on continuous movement"),
        ] = None,
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Set stand movement with either an angle or a movement type  # noqa: E501


        :param stand_movement: Stand Movement (required)
        :type stand_movement: StandMovement
        :param slow_speed: Use slow speed on continuous movement
        :type slow_speed: bool
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["stand_movement", "slow_speed"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_stand_movement" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("slow_speed") is not None:  # noqa: E501
            _query_params.append(("slow-speed", _params["slow_speed"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["stand_movement"] is not None:
            _body_params = _params["stand_movement"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/stand/move",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field
except ImportError:
    from pydantic import BaseModel, Field

from mozart_api.models.room_compensation_debug import RoomCompensationDebug
from mozart_api.models.room_compensation_enabled import RoomCompensationEnabled
from mozart_api.models.room_compensation_result import RoomCompensationResult
from mozart_api.models.room_compensation_state import RoomCompensationState
from mozart_api.models.room_compensation_type import RoomCompensationType
from mozart_api.models.room_compensation_version import RoomCompensationVersion


class RoomCompensationInfo(BaseModel):
    """
    RoomCompensationInfo
    """

    debug: Optional[RoomCompensationDebug] = None
    enabled: Optional[RoomCompensationEnabled] = None
    last_result: Optional[RoomCompensationResult] = Field(
        default=None, alias="lastResult"
    )
    last_run: Optional[RoomCompensationState] = Field(default=None, alias="lastRun")
    type: Optional[RoomCompensationType] = None
    version: Optional[RoomCompensationVersion] = None
    __properties = ["debug", "enabled", "lastResult", "lastRun", "type", "version"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RoomCompensationInfo:
        """Create an instance of RoomCompensationInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of debug
        if self.debug:
            _dict["debug"] = self.debug.to_dict()
        # override the default output from pydantic by calling `to_dict()` of enabled
        if self.enabled:
            _dict["enabled"] = self.enabled.to_dict()
        # override the default output from pydantic by calling `to_dict()` of last_result
        if self.last_result:
            _dict["lastResult"] = self.last_result.to_dict()
        # override the default output from pydantic by calling `to_dict()` of last_run
        if self.last_run:
            _dict["lastRun"] = self.last_run.to_dict()
        # override the default output from pydantic by calling `to_dict()` of type
        if self.type:
            _dict["type"] = self.type.to_dict()
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict["version"] = self.version.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RoomCompensationInfo:
        """Create an instance of RoomCompensationInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RoomCompensationInfo.parse_obj(obj)

        _obj = RoomCompensationInfo.parse_obj(
            {
                "debug": RoomCompensationDebug.from_dict(obj.get("debug"))
                if obj.get("debug") is not None
                else None,
                "enabled": RoomCompensationEnabled.from_dict(obj.get("enabled"))
                if obj.get("enabled") is not None
                else None,
                "last_result": RoomCompensationResult.from_dict(obj.get("lastResult"))
                if obj.get("lastResult") is not None
                else None,
                "last_run": RoomCompensationState.from_dict(obj.get("lastRun"))
                if obj.get("lastRun") is not None
                else None,
                "type": RoomCompensationType.from_dict(obj.get("type"))
                if obj.get("type") is not None
                else None,
                "version": RoomCompensationVersion.from_dict(obj.get("version"))
                if obj.get("version") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator
except ImportError:
    from pydantic import BaseModel, Field, StrictInt, StrictStr, validator

from mozart_api.models.beo_shape_status_sound_profile import BeoShapeStatusSoundProfile


class BeoShapeStatus(BaseModel):
    """
    Status  # noqa: E501
    """

    amplifier_tiles_configured: StrictInt = Field(
        default=...,
        alias="amplifierTilesConfigured",
        description="Number of amplifier Shape titles currently configured",
    )
    amplifier_tiles_online: StrictInt = Field(
        default=...,
        alias="amplifierTilesOnline",
        description="Number of amplifier Shape tiles currently connected",
    )
    design_id: Optional[StrictStr] = Field(default=None, alias="designId")
    design_name: Optional[StrictStr] = Field(default=None, alias="designName")
    sound_profile: Optional[BeoShapeStatusSoundProfile] = Field(
        default=None, alias="soundProfile"
    )
    speaker_tiles_configured: StrictInt = Field(
        default=...,
        alias="speakerTilesConfigured",
        description="Number of speaker Shape titles currently configured",
    )
    state: StrictStr = Field(...)
    __properties = [
        "amplifierTilesConfigured",
        "amplifierTilesOnline",
        "designId",
        "designName",
        "soundProfile",
        "speakerTilesConfigured",
        "state",
    ]

    @validator("state")
    def state_validate_enum(cls, value):
        """Validates the enum"""
        if value not in (
            "notUsed",
            "updatingFirmware",
            "initializing",
            "errorHardwareConfig",
            "ready",
        ):
            raise ValueError(
                "must be one of enum values ('notUsed', 'updatingFirmware', 'initializing', 'errorHardwareConfig', 'ready')"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BeoShapeStatus:
        """Create an instance of BeoShapeStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of sound_profile
        if self.sound_profile:
            _dict["soundProfile"] = self.sound_profile.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BeoShapeStatus:
        """Create an instance of BeoShapeStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BeoShapeStatus.parse_obj(obj)

        _obj = BeoShapeStatus.parse_obj(
            {
                "amplifier_tiles_configured": obj.get("amplifierTilesConfigured"),
                "amplifier_tiles_online": obj.get("amplifierTilesOnline"),
                "design_id": obj.get("designId"),
                "design_name": obj.get("designName"),
                "sound_profile": BeoShapeStatusSoundProfile.from_dict(
                    obj.get("soundProfile")
                )
                if obj.get("soundProfile") is not None
                else None,
                "speaker_tiles_configured": obj.get("speakerTilesConfigured"),
                "state": obj.get("state"),
            }
        )
        return _obj
